// Programmer: Maura Dailey

// Class: CSE 120
// Section: 1
// Date: 02/04/02

// Brief: Prototypes and definitions of functions for class StrType

#include "String.h"
#include <cctype>
#include <fstream>
#include <iostream>
#include <cstring>

// Prototypes of auxiliary functions.
// Note: If skip is true, non-allowable leading characters are skipped
//       If end-of-file is encountered while skipping characters, the
//       empty string is returned.
//       If the number of allowable characters exceeds MAX, the rest are
//       read and discarded.

void  GetAlphaNum(bool skip, char letters[]);
// Post: letters array contains only alphabetic characters and numbers.

void  GetAlpha(bool skip, char letters[]);
// Post: letters array contains only alphabetic characters.

void  GetNonWhite(bool skip, char letters[]);
// Post: letters array contains only non-white space characters.

void  GetTilNew(bool skip, char letters[]);
// Post: letters array contains everything up to the newline character.

void GetAlphaNumFile(bool skip, char letters[], std::ifstream& inFile);
// Post: letters array contains only alphabetic characters and numbers.

void GetAlphaFile(bool skip, char letters[], std::ifstream& inFile);
// Post: letters array contains only alphabetic characters.

void GetNonWhiteFile(bool skip, char letters[], std::ifstream& inFile);
// Post: letters array contains only non-white space characters.

void GetTilNewFile(bool skip, char letters[], std::ifstream& inFile);
// Post: letters array contains everything up to the newline character.

// Function used to call one of four other functions. Function to call
// is determined by the parameter variable charsAllowed.

void StrType::GetString(bool skip, InType charsAllowed)
{
    switch (charsAllowed)
    {
    	case ALPHA_NUM : GetAlphaNum(skip, letters);
			 break;
	case ALPHA     : GetAlpha(skip, letters);
			 break;
	case NON_WHITE : GetNonWhite(skip, letters);
			 break;
        case NOT_NEW   : GetTilNew(skip, letters);
			 break;
    }
}

// Function used to call one of four other functions. Function to call
// is determined by the parameter variable charsAllowed.

void StrType::GetStringFile(bool skip, InType charsAllowed, std::ifstream& inFile)
{
    switch (charsAllowed)
    {
       case ALPHA_NUM : GetAlphaNumFile(skip, letters, inFile);
                        break;
       case ALPHA     : GetAlphaFile(skip, letters, inFile);
                        break;
       case NON_WHITE : GetNonWhiteFile(skip, letters, inFile);
                        break;
       case NOT_NEW   : GetTilNewFile(skip, letters, inFile);
                        break;
    }
}

void  GetAlphaNum(bool skip, char letters[])
// Post: If skip is true, non-alphanumeric letters are skipped.
//       Alphanumeric characters are read and stored until a
//       non-alphanumeric character is read or MAX_CHARS characters
//       have been stored. If the stream is not in the fail state,
//       the last character read was a non-alphanumeric character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    cin.get(letter);
    while (!isalnum(letter) && cin)
      cin.get(letter);
  }
  else
    cin.get(letter);
  if (!cin || !isalnum(letter))
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
	cin.get(letter);
    } while (isalnum(letter) && cin && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
          do
      {
        cin.get(letter);
      } while (isalnum(letter) && cin);
  }
}

void  GetAlpha(bool skip, char letters[])
// Post: If skip is true, non-alphanumeric letters are skipped.
//       Alphabetic characters are read and stored until a
//       non-alphabetic character is read or MAX_CHARS characters
//       have been stored. If the stream is not in the fail state,
//       the last character read was a non-alphabetic character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    cin.get(letter);
    while (!isalpha(letter) && cin)
      cin.get(letter);
  }
  else
    cin.get(letter);
  if (!cin || !isalpha(letter))
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
	cin.get(letter);
    } while (isalpha(letter) && cin && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
      do
      {
        cin.get(letter);
      } while (isalpha(letter) && cin);
  }
}

void  GetNonWhite(bool skip, char letters[])
// Post: If skip is true, whitespace characters are skipped.
//       All characters are read and stored until a whitespace
//       character is read or MAX_CHARS characters have been
//       stored. If the stream is not in the fail state, the
//       last character read was a whitespace character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    cin.get(letter);
    while (cin && isspace(letter))
      cin.get(letter);
  }
  else
    cin.get(letter);
  if (!cin || isspace(letter))
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
	cin.get(letter);
    } while (!isspace(letter) && cin && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
      do
      {
        cin.get(letter);
      } while (!isspace(letter) && cin);
  }
}

void  GetTilNew(bool skip, char letters[])
// Post: If skip is true, newline characters are skipped.
//       All characters are read and stored until a newline
//       character is read or MAX_CHARS characters have been
//       stored. If the stream is not in the fail state, the
//       last character read was a newline character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    cin.get(letter);
    while ((letter == '\n') && cin)
      cin.get(letter);
  }
  else
    cin.get(letter);
  if (!cin || letter == '\n')
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
      cin.get(letter);
    } while ((letter != '\n') && cin && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
    do
    {
      cin.get(letter);
    } while ((letter == '\n') && cin);
  }
}

void  GetAlphaNumFile(bool skip, char letters[], std::ifstream& inFile)
// Post: If skip is true, non-alphanumeric letters are skipped.
//       Alphanumeric characters are read and stored until a
//       non-alphanumeric character is read or MAX_CHARS characters
//       have been stored. If the stream is not in the fail state,
//       the last character read was a non-alphanumeric character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    inFile.get(letter);
    while (!isalnum(letter) && inFile)
      inFile.get(letter);
  }
  else
    inFile.get(letter);
  if (!inFile || !isalnum(letter))
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
	inFile.get(letter);
    } while (isalnum(letter) && inFile && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
      do
      {
        inFile.get(letter);
      } while (isalnum(letter) && inFile);
  }
}

void GetAlphaFile(bool skip, char letters[], std::ifstream& inFile)
// Post: If skip is true, non-alphanumeric letters are skipped.
//       Alphabetic characters are read and stored until a
//       non-alphabetic character is read or MAX_CHARS characters
//       have been stored. If the stream is not in the fail state,
//       the last character read was a non-alphabetic character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    inFile.get(letter);
    while (!isalpha(letter) && inFile)
      inFile.get(letter);
  }
  else
    inFile.get(letter);
  if (!inFile || !isalpha(letter))
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
	inFile.get(letter);
    } while (isalpha(letter) && inFile && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
      do
      {
        inFile.get(letter);
      } while (isalpha(letter) && inFile);
  }
}

void GetNonWhiteFile(bool skip, char letters[], std::ifstream& inFile)
// Post: If skip is true, whitespace characters are skipped.
//       All characters are read and stored until a whitespace
//       character is read or MAX_CHARS characters have been
//       stored. If the stream is not in the fail state, the
//       last character read was a whitespace character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    inFile.get(letter);
    while (isspace(letter) && inFile)
      inFile.get(letter);
  }
  else
    inFile.get(letter);
  if (!inFile || isspace(letter))
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
	inFile.get(letter);
    } while (!isspace(letter) && inFile && (count < MAX_CHARS));
    letters[count] = '\0';
    // Skip extra characters if necessary.
    if (count == MAX_CHARS)
      do
      {
        inFile.get(letter);
      } while (!isspace(letter) && inFile);
  }
}

void GetTilNewFile(bool skip, char letters[], std::ifstream& inFile)
// Post: If skip is true, newline characters are skipped.
//       All characters are read and stored until a newline
//       character is read or MAX_CHARS characters have been
//       stored. If the stream is not in the fail state, the
//       last character read was a newline character.

{
  using namespace std;
  char letter;
  int count = 0;

  if (skip)
  {
    inFile.get(letter);
    while ((letter == '\n') && inFile)
      inFile.get(letter);
  }
  else
    inFile.get(letter);
  if (!inFile || letter == '\n')
    letters[0] = '\0';
  else
  {
    do
    {
      letters[count] = letter;
      count++;
      inFile.get(letter);
     } while ((letter != '\n') && inFile && (count < MAX_CHARS));
     letters[count] = '\0';
     // Skip extra characters if necessary.
     if (count == MAX_CHARS)
     do
     {
       inFile.get(letter);
     } while ((letter == '\n') && inFile);
  }
}

void StrType::CopyString(StrType& newString)
{
  strcpy(newString.letters, letters);
}

void StrType::PrintToScreen(bool newLine)
{
  using namespace std;
  if (newLine)
    cout  << endl;
  cout  << letters;
}

void StrType::PrintToFile(bool newLine, std::ofstream& outFile)
{
  if (newLine)
    outFile  << std::endl;
  outFile  << letters;
}

void StrType::MakeEmpty()
{
  letters[0] = '\0';
}

int StrType::LengthIs()
{
  return strlen(letters);
}
